from typing import List, Optional
from enum import Enum
from dataclasses import dataclass

@dataclass
class FilterOptions:
    price_min: Optional[float] = None
    price_max: Optional[float] = None
    market_cap_min: Optional[float] = None
    market_cap_max: Optional[float] = None
    volume_24h_min: Optional[float] = None
    volume_24h_max: Optional[float] = None
    circulating_supply_min: Optional[float] = None
    circulating_supply_max: Optional[float] = None
    percent_change_24h_min: Optional[float] = None
    percent_change_24h_max: Optional[float] = None
    tags: Optional[List[str]] = None


class AuxFields(Enum):
    NUM_MARKET_PAIRS = "num_market_pairs"
    CMC_RANK = "cmc_rank"
    DATE_ADDED = "date_added"
    TAGS = "tags"
    PLATFORM = "platform"
    MAX_SUPPLY = "max_supply"
    TOTAL_SUPPLY = "total_supply"
    MARKET_CAP_BY_TOTAL_SUPPLY = "market_cap_by_total_supply"
    VOLUME_24H_REPORTED = "volume_24h_reported"
    VOLUME_7D = "volume_7d"
    VOLUME_7D_REPORTED = "volume_7d_reported"
    VOLUME_30D = "volume_30d"
    VOLUME_30D_REPORTED = "volume_30d_reported"
    IS_MARKET_CAP_INCLUDED = "is_market_cap_included_in_calc"

class SortDir(Enum):
	ASC = "asc"
	DESC = "desc"

class SortOption(Enum):
    MARKET_CAP = "market_cap"
    MARKET_CAP_STRICT = "market_cap_strict"
    NAME = "name"
    SYMBOL = "symbol"
    DATE_ADDED = "date_added"
    PRICE = "price"
    CIRCULATING_SUPPLY = "circulating_supply"
    TOTAL_SUPPLY = "total_supply"
    MAX_SUPPLY = "max_supply"
    NUM_MARKET_PAIRS = "num_market_pairs"
    MARKET_CAP_BY_TOTAL_SUPPLY_STRICT = "market_cap_by_total_supply_strict"
    VOLUME_24H = "volume_24h"
    VOLUME_7D = "volume_7d"
    VOLUME_30D = "volume_30d"
    PERCENT_CHANGE_1H = "percent_change_1h"
    PERCENT_CHANGE_24H = "percent_change_24h"
    PERCENT_CHANGE_7D = "percent_change_7d"




from dataclasses import dataclass, field
from typing import List, Optional, Dict
import time
import datetime
import json

from dataclasses import dataclass, field
from typing import Dict, List, Optional
import datetime
from dateutil import parser

from dataclasses import dataclass
from typing import Optional
import datetime

@dataclass
class Quote:
    base_currency: str
    price: float
    volume_24h: float
    percent_change_1h: float
    percent_change_24h: float
    percent_change_7d: float
    percent_change_30d: float
    market_cap: float
    last_updated: datetime.datetime

    # Now all optional/default parameters follow
    volume_change_24h: float = 0.0
    percent_change_60d: float = 0.0
    percent_change_90d: float = 0.0
    market_cap_dominance: float = 0.0
    fully_diluted_market_cap: float = 0.0
    tvl: Optional[float] = None
    volume_30d: Optional[float] = None
    volume_30d_reported: Optional[float] = None
    volume_24h_reported: Optional[float] = None
    volume_7d_reported: Optional[float] = None
    market_cap_by_total_supply: Optional[float] = None
    volume_7d: Optional[float] = None
    total_supply: Optional[float] = None
    circulating_supply: Optional[float] = None

    @staticmethod
    def from_dict(currency: str, dct_quote_data: Dict) -> 'Quote':

        # insure integers are handled as floats (so 1 becomes 1.0)
        dct_quote_data['price'] = float(dct_quote_data['price'])
        dct_quote_data['market_cap'] = float(dct_quote_data['market_cap'])


        # Handle both 'last_updated' and 'timestamp' for the last_updated field
        if 'last_updated' in dct_quote_data:
            last_updated_str = dct_quote_data['last_updated']
        elif 'timestamp' in dct_quote_data:
            last_updated_str = dct_quote_data['timestamp']
        else:
            raise ValueError("Payload must contain either 'last_updated' or 'timestamp' field.")

        # Remove both possible keys to avoid errors in the constructor
        dct_quote_data.pop('last_updated', None)
        dct_quote_data.pop('timestamp', None)

        last_updated = parser.parse(last_updated_str)
        
        return Quote(base_currency=currency, last_updated=last_updated, **dct_quote_data)


@dataclass
class TokenState:
    id: int
    name: str
    symbol: str
    last_updated: datetime.datetime
    quote_map: Dict[str, Quote]

    timestamp: int = int(time.time())
    infinite_supply: bool = None
    slug: Optional[str] = None
    num_market_pairs: Optional[int] = None
    date_added: Optional[datetime.datetime] = None
    tags: Optional[List[str]] = None
    max_supply: Optional[int] = None
    circulating_supply: Optional[int] = None
    total_supply: Optional[float] = None
    platform: Optional[str] = None
    cmc_rank: Optional[int] = None
    self_reported_circulating_supply: Optional[int] = None
    self_reported_market_cap: Optional[float] = None
    tvl_ratio: Optional[float] = None
    is_market_cap_included_in_calc: Optional[bool] = None
    is_active: Optional[bool] = None
    is_fiat: Optional[bool] = None


    @staticmethod
    def from_dict(data: Dict) -> 'TokenState':
        data = data.copy()

        # Convert 'is_market_cap_included_in_calc' from 0/1 to False/True
        if 'is_market_cap_included_in_calc' in data:
            data['is_market_cap_included_in_calc'] = bool(data['is_market_cap_included_in_calc'])

        quote_map = {}
        dct_quote_data = data.pop('quote')
        for currency, dct_quote_data in dct_quote_data.items():
            quote_map[currency] = Quote.from_dict(currency, dct_quote_data)
        data['quote_map'] = quote_map

        # Set optional attributes to None if not present in the data
        optional_fields = [
            'num_market_pairs', 'date_added', 'tags', 'max_supply', 'circulating_supply',
            'total_supply', 'platform', 'cmc_rank', 'self_reported_circulating_supply',
            'self_reported_market_cap', 'tvl_ratio'
        ]

        for attr_name in optional_fields:
            if attr_name not in data:
                data[attr_name] = None

        return TokenState(**data)





from typing import Optional, List
import json
from dateutil import parser
import time
from pprint import pprint

from coinmarketcap.types.token_state import TokenState, Quote

def _validate_interval(interval: str) -> None:
    # Define allowed calendar year and time constants
    calendar_intervals = {"hourly", "daily", "weekly", "monthly", "yearly"}

    # Define allowed relative time intervals
    relative_intervals = {
        "5m", "10m", "15m", "30m", "45m",
        "1h", "2h", "3h", "4h", "6h", "12h",
        "1d", "2d", "3d", "7d", "14d", "15d", "30d", "60d", "90d", "365d"
    }

    # Check if the interval is in one of the allowed sets
    if interval not in calendar_intervals and interval not in relative_intervals:
        # If not, raise a ValueError with a message about the invalid interval
        raise ValueError(f"Invalid interval: '{interval}'. Please provide a valid interval.")

def _quotes_historical_v2(market,
						  id: Optional[str] = None,
						  ticker: Optional[str] = None,
						  timestamp_start: Optional[int] = int(time.time()) - 60*60*24,
						  timestamp_end: Optional[int] = int(time.time()),
						  interval: str = 'hourly',
						  convert: List[str] = ['USD']) -> List[TokenState]:

	if not id and not ticker:
		raise ValueError('Either id or ticker must be provided')

	# Check if the start timestamp is greater than the end timestamp
	if timestamp_start > timestamp_end:
		raise ValueError('The start timestamp occurr before than the end timestamp')

	# Check if the interval is valid
	_validate_interval(interval)

	# validate convert
	if (len(convert) > 3):
		raise ValueError('The convert list must have a maximum of 3 elements')

	params = {
		'time_start': timestamp_start,
		'time_end': timestamp_end,
		'interval': interval,
		'convert': ','.join(convert)
	}

	# use id if provided, otherwise use ticker
	if id:
		params['id'] = id
	else:
		params['symbol'] = ticker
		
	response = market._request('v2/cryptocurrency/quotes/historical', params=params)

	lst_token_states = []

	if id:
		# if we are querying by id, we get a simpler (although not completely simple)
		# structure to parse
		dct_quote_summary = response['data']
	else:
		# if we query by ticker we get a differeint weird structure, we have to 
		# drill down into the quotes object for our ticker, we call this the quote 
	    # summary because it's the quotes, plus some extra
		# meta data we can extract for the TokenState object
		dct_quote_summary = response['data'][ticker][0]

	# and we also get some general meta-data that can go into the TokenState object
	id = dct_quote_summary['id']
	name = dct_quote_summary['name']
	symbol = dct_quote_summary['symbol']
	is_active = dct_quote_summary['is_active'] == 1
	is_fiat = dct_quote_summary['is_fiat'] == 1

	# and the quotes themselves, still wrapped up in a list of convoluted stuff
	lst_quotes = dct_quote_summary['quotes']

	# for each quote block, we can create a token state
	for dct_quote_block in lst_quotes:

		# Parse the timestamp string into a datetime object
		timestamp_dt = parser.parse(dct_quote_block['timestamp'])

		# create a token state, the quotes are empty for now
		token_state = TokenState(
			id=id,
			name=name,
			symbol=symbol,
			last_updated=timestamp_dt,
			timestamp=int(timestamp_dt.timestamp()),
			is_active=is_active,
			quote_map={},
			is_fiat=is_fiat)

		# init each quote object and add it to the tokenstate
		for base_currency, dct_quote_data in dct_quote_block['quote'].items():
			quote = Quote.from_dict(base_currency, dct_quote_data)
			token_state.quote_map[base_currency] = quote
					
		lst_token_states.append(token_state)

	return lst_token_states



import pytest
from datetime import datetime
import os
import time

from coinmarketcap import Market
from coinmarketcap.types.token_state import Quote
from coinmarketcap.v1.cryptocurrency.listings.common import SortOption, AuxFields, SortDir, FilterOptions

@pytest.fixture
def coinmarketcap_instance():
    # You can initialize your CoinMarketCap instance with your API key here if needed
    api_key=API_KEY = os.environ.get('COIN_MARKET_CAP_API_KEY')
    coinmarketcap_instance = Market(api_key=api_key, debug_mode=True)
    yield coinmarketcap_instance


def test_cryptocurrency_quotes_historical_with_id(coinmarketcap_instance):

    timestamp_now = int(time.time())
    timestamp_1_day_ago = timestamp_now - 60*60*24

    # Make the API call
    token_states = coinmarketcap_instance.quotes_historical(
        id="1",
        timestamp_start=timestamp_1_day_ago,
        timestamp_end=timestamp_now,
        interval='1h',
        convert=['USD', 'BTC']
    )

    # Check if the response is a list and contains at least one item
    assert isinstance(token_states, list)
    assert len(token_states) >= 1

    # Check the attributes of the first token state
    token_state = token_states[0]
    assert isinstance(token_state.id, int)
    assert isinstance(token_state.name, str)
    assert isinstance(token_state.symbol, str)
    assert isinstance(token_state.last_updated, datetime)
    assert isinstance(token_state.timestamp, int)
    assert isinstance(token_state.is_active, bool)
    assert isinstance(token_state.is_fiat, bool)
    assert isinstance(token_state.quote_map, dict)

    # Check the attributes of the USD quote
    quote = token_state.quote_map['USD']
    assert isinstance(quote.price, float)
    assert isinstance(quote.volume_24h, float)
    assert isinstance(quote.percent_change_1h, float)
    assert isinstance(quote.percent_change_24h, float)
    assert isinstance(quote.percent_change_7d, float)
    assert isinstance(quote.market_cap, float)
    assert isinstance(quote.last_updated, datetime)

    # Check the attributes of the BTC quote
    quote = token_state.quote_map['BTC']
    assert isinstance(quote.price, float)
    assert isinstance(quote.volume_24h, float)
    assert isinstance(quote.percent_change_1h, float)
    assert isinstance(quote.percent_change_24h, float)
    assert isinstance(quote.percent_change_7d, float)
    assert isinstance(quote.market_cap, float)
    assert isinstance(quote.last_updated, datetime)


def test_cryptocurrency_quotes_historical_with_ticker(coinmarketcap_instance):
    
    timestamp_now = int(time.time())
    timestamp_1_day_ago = timestamp_now - 60*60*24
    
    # Make the API call
    token_states = coinmarketcap_instance.quotes_historical(
        ticker='ETH',
        timestamp_start=timestamp_1_day_ago,
        timestamp_end=timestamp_now,
        interval='1h',
        convert=['USD', 'BTC']
    )

    # Check if the response is a list and contains at least one item
    assert isinstance(token_states, list)
    assert len(token_states) >= 1

    # Check the attributes of the first token state
    token_state = token_states[0]
    assert isinstance(token_state.id, int)
    assert isinstance(token_state.name, str)
    assert isinstance(token_state.symbol, str)
    assert isinstance(token_state.last_updated, datetime)
    assert isinstance(token_state.timestamp, int)
    assert isinstance(token_state.is_active, bool)
    assert isinstance(token_state.is_fiat, bool)
    assert isinstance(token_state.quote_map, dict)

    # Check the attributes of the USD quote
    quote = token_state.quote_map['USD']
    assert isinstance(quote.price, float)
    assert isinstance(quote.volume_24h, float)
    assert isinstance(quote.percent_change_1h, float)
    assert isinstance(quote.percent_change_24h, float)
    assert isinstance(quote.percent_change_7d, float)
    assert isinstance(quote.market_cap, float)
    assert isinstance(quote.last_updated, datetime)

    # Check the attributes of the BTC quote
    quote = token_state.quote_map['BTC']
    assert isinstance(quote.price, float)
    assert isinstance(quote.volume_24h, float)
    assert isinstance(quote.percent_change_1h, float)
    assert isinstance(quote.percent_change_24h, float)
    assert isinstance(quote.percent_change_7d, float)
    assert isinstance(quote.market_cap, float)
    assert isinstance(quote.last_updated, datetime)


def test_listings_latest(coinmarketcap_instance):
    # Define the filter options
    filter = FilterOptions(
        price_min=10,
        price_max=100,
        volume_24h_min=1000000,
        percent_change_24h_min=-5,
        tags=["defi"]
    )

    # Define the aux fields
    aux_fields = [
        AuxFields.NUM_MARKET_PAIRS,
        AuxFields.PLATFORM,
        AuxFields.TOTAL_SUPPLY,
        AuxFields.TAGS,
        AuxFields.VOLUME_30D, 
        AuxFields.CMC_RANK, 
        AuxFields.DATE_ADDED, 
        AuxFields.IS_MARKET_CAP_INCLUDED, 
        AuxFields.MARKET_CAP_BY_TOTAL_SUPPLY, 
        AuxFields.MAX_SUPPLY,
        AuxFields.VOLUME_30D_REPORTED,
        AuxFields.VOLUME_30D, 
        AuxFields.VOLUME_24H_REPORTED, 
        AuxFields.VOLUME_7D,
        AuxFields.VOLUME_7D_REPORTED
    ]

    # Make the API call
    tokens = coinmarketcap_instance.listings_latest(
        sort_by=SortOption.MARKET_CAP,
        sort_dir=SortDir.DESC,
        convert=['USD'],
        limit=1,
        filters=filter,
        aux_fields=aux_fields
    )

    # Check if the response is a list and contains at least one item
    assert isinstance(tokens, list)
    assert len(tokens) >= 1

    # Check the attributes of the first token
    token = tokens[0]
    assert isinstance(token.id, int)
    assert isinstance(token.name, str)
    assert isinstance(token.symbol, str)
    assert isinstance(token.slug, str)
    assert isinstance(token.infinite_supply, bool)
    assert isinstance(token.quote_map, dict)

    # Check optional attributes (can be None)
    assert token.num_market_pairs is None or isinstance(token.num_market_pairs, int)
    assert token.tags is None or isinstance(token.tags, list)
    assert token.max_supply is None or isinstance(token.max_supply, int)
    assert token.circulating_supply is None or isinstance(token.circulating_supply, int)
    assert token.total_supply is None or isinstance(token.total_supply, float)
    assert token.platform is None or isinstance(token.platform, str)
    assert token.cmc_rank is None or isinstance(token.cmc_rank, int)
    assert token.self_reported_circulating_supply is None or isinstance(token.self_reported_circulating_supply, int)
    assert token.self_reported_market_cap is None or isinstance(token.self_reported_market_cap, float)
    assert token.tvl_ratio is None or isinstance(token.tvl_ratio, float)
    assert token.is_market_cap_included_in_calc is None or isinstance(token.is_market_cap_included_in_calc, bool)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__title__ = 'coinmarketcap'
__version__ = '0.1.2'
__author__ = 'Jason Byteforgia'
__repo__ = 'zhttps://github.com/jmazzahacks/byteforge-coinmarketcap'
__license__ = 'Apache v2.0 License'

from .core import Market
from .core import ServerException
from .core import SortOption
from .core import SortDir
from .core import FilterOptions
from .core import AuxFields